# Copyright (C) 2023-2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

"""Loader in charge of injecting tree at a specific revision."""

from datetime import datetime
import os
from pathlib import Path
import tempfile
from typing import Iterator, List, Optional

from swh.loader.core.loader import BaseDirectoryLoader
from swh.loader.svn.svn_repo import SvnRepo, get_svn_repo
from swh.model.model import Snapshot, SnapshotBranch, SnapshotTargetType


class SvnExportLoader(BaseDirectoryLoader):
    """Load a svn tree at a specific svn revision into the swh archive.

    It is also possible to load a subset of the source tree by explicitly
    specifying the sub-paths to export in the ``svn_paths`` optional parameter.

    If the origin URL should be different from the subversion URL, the latter
    can be provided using the optional ``svn_url`` parameter.

    The output snapshot is of the form:

    .. code::

       id: <bytes>
       branches:
         HEAD:
           target_type: alias
           target: rev_<svn-revision>
         rev_<svn-revision>:
           target_type: directory
           target: <directory-id>

    """

    visit_type = "svn-export"

    def __init__(
        self,
        *args,
        svn_paths: Optional[List[str]] = None,
        svn_url: Optional[str] = None,
        **kwargs,
    ):
        self.svn_revision = int(kwargs.pop("ref"))
        self.svn_paths = svn_paths
        super().__init__(*args, **kwargs)
        self.svn_url = svn_url
        if self.svn_url is None:
            self.svn_url = self.origin.url
        self.svnrepo: Optional[SvnRepo] = None

    def prepare(self) -> None:
        self.svnrepo = get_svn_repo(self.svn_url, revision=self.svn_revision)
        super().prepare()

    def cleanup(self) -> None:
        """Clean up any intermediary fs."""
        if self.svnrepo:
            self.svnrepo.clean_fs()

    def fetch_artifact(self) -> Iterator[Path]:
        """Prepare the svn local repository checkout at a given commit/tag."""
        assert self.svnrepo is not None
        if self.svn_paths is None:
            _, local_url = self.svnrepo.export_temporary(self.svn_revision)
            yield Path(local_url.decode())
        else:
            assert self.svn_url is not None
            self.log.debug(
                "Exporting from the svn source tree rooted at %s@%s the sub-paths: %s",
                self.svn_url,
                self.svn_revision,
                ", ".join(self.svn_paths),
            )
            with tempfile.TemporaryDirectory(
                suffix="-" + datetime.now().isoformat()
            ) as tmp_dir:
                for svn_path in self.svn_paths:
                    svn_url = os.path.join(self.svn_url, svn_path.strip("/"))
                    export_path = os.path.join(tmp_dir, svn_path.strip("/"))
                    os.makedirs("/".join(export_path.split("/")[:-1]), exist_ok=True)
                    self.svnrepo.export(
                        svn_url,
                        export_path,
                        rev=int(self.svn_revision),
                        remove_dest_path=False,
                        overwrite=True,
                        ignore_externals=True,
                        ignore_keywords=True,
                    )
                yield Path(tmp_dir)

    def build_snapshot(self) -> Snapshot:
        """Build snapshot without losing the svn revision context."""
        assert self.directory is not None
        branch_name = f"rev_{self.svn_revision}".encode()
        return Snapshot(
            branches={
                b"HEAD": SnapshotBranch(
                    target_type=SnapshotTargetType.ALIAS,
                    target=branch_name,
                ),
                branch_name: SnapshotBranch(
                    target_type=SnapshotTargetType.DIRECTORY,
                    target=self.directory.hash,
                ),
            }
        )



# Copyright (C) 2023  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

from swh.loader.tests import assert_module_tasks_are_scheduler_ready


def test_tasks_loader_visit_type_match_task_name():
    import swh.loader.svn

    assert_module_tasks_are_scheduler_ready([swh.loader.svn])



# Copyright (C) 2019-2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

from typing import Any, Dict


def register() -> Dict[str, Any]:
    from swh.loader.svn.loader import SvnLoaderFromRemoteDump

    return {
        "task_modules": [f"{__name__}.tasks"],
        "loader": SvnLoaderFromRemoteDump,
    }


def register_export() -> Dict[str, Any]:
    from swh.loader.svn.directory import SvnExportLoader

    return {
        "task_modules": [],
        "loader": SvnExportLoader,
    }


def register_no_dump() -> Dict[str, Any]:
    from swh.loader.svn.loader import SvnLoader

    return {
        "task_modules": [f"{__name__}.tasks"],
        "loader": SvnLoader,
    }


def register_from_dump() -> Dict[str, Any]:
    from swh.loader.svn.loader import SvnLoaderFromDump

    return {
        "task_modules": [f"{__name__}.tasks"],
        "loader": SvnLoaderFromDump,
    }



# Copyright (C) 2023-2025  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import os
from pathlib import Path

import pytest

from swh.core.nar import Nar
from swh.loader.svn.directory import SvnExportLoader
from swh.loader.svn.svn_repo import get_svn_repo
from swh.loader.tests import (
    assert_last_visit_matches,
    fetch_extids_from_checksums,
    get_stats,
    prepare_repository_from_archive,
)


def compute_nar_hash_for_rev(repo_url: str, rev: int, hash_name: str = "sha256") -> str:
    """Compute the Nar hashes of the svn tree at the revision 'rev'."""
    svn_repo = get_svn_repo(repo_url, revision=rev)
    assert svn_repo is not None
    _, export_dir = svn_repo.export_temporary(rev)

    nar = Nar(hash_names=[hash_name])
    nar.serialize(Path(export_dir.decode()))
    return nar.hexdigest()[hash_name]


def test_loader_svn_directory(swh_storage, datadir, tmp_path):
    """Loading a svn tree with proper nar checksums should be eventful"""
    archive_name = "pkg-gourmet"
    archive_path = os.path.join(datadir, f"{archive_name}.tgz")
    repo_url = prepare_repository_from_archive(
        archive_path, archive_name, tmp_path=tmp_path
    )
    svn_revision = 5
    checksums = {"sha256": compute_nar_hash_for_rev(repo_url, svn_revision)}

    loader = SvnExportLoader(
        swh_storage,
        repo_url,
        ref=svn_revision,
        checksum_layout="nar",
        checksums=checksums,
    )

    actual_result = loader.load()

    assert actual_result == {"status": "eventful"}

    actual_visit = assert_last_visit_matches(
        swh_storage,
        repo_url,
        status="full",
        type="svn-export",
    )

    snapshot = swh_storage.snapshot_get(actual_visit.snapshot)
    assert snapshot is not None

    branches = snapshot["branches"].keys()
    expected_branch = f"rev_{svn_revision}".encode()
    assert set(branches) == {b"HEAD", expected_branch}

    assert get_stats(swh_storage) == {
        "content": 18,
        "directory": 6,
        "origin": 1,
        "origin_visit": 1,
        "release": 0,
        "revision": 0,
        "skipped_content": 0,
        "snapshot": 1,
    }

    # Ensure the extids got stored as well
    extids = fetch_extids_from_checksums(
        loader.storage,
        checksum_layout="nar",
        checksums=checksums,
        extid_version=loader.extid_version,
    )
    assert len(extids) == len(checksums)

    # Another run on the same svn directory should be uneventful
    loader2 = SvnExportLoader(
        swh_storage,
        repo_url,
        ref=svn_revision,
        checksum_layout="nar",
        checksums=checksums,
    )
    actual_result2 = loader2.load()
    assert actual_result2 == {"status": "uneventful"}


@pytest.mark.parametrize(
    "use_custom_origin_url",
    [False, True],
    ids=["origin_url == svn_url", "origin_url != svn_url"],
)
def test_loader_svn_directory_sub_paths(
    swh_storage, datadir, tmp_path, use_custom_origin_url
):
    """Loading a subset of a svn tree with proper nar checksums should be eventful"""
    archive_name = "pkg-gourmet"
    archive_path = os.path.join(datadir, f"{archive_name}.tgz")
    repo_url = prepare_repository_from_archive(
        archive_path, archive_name, tmp_path=tmp_path
    )
    svn_paths = ["gourmet/trunk/debian/gourmet.1", "gourmet/trunk/debian/patches"]
    svn_revision = 5
    checksum_layout = "nar"
    checksums = {
        "sha256": "21e9553da2f8ae27d6b9ae87f509b0233fc6edbabc1099c31b90e1dec2cbb618"
    }

    origin_url = (
        f"{repo_url}?nar=sha256-{checksums['sha256']}"
        if use_custom_origin_url
        else repo_url
    )
    svn_url = repo_url if use_custom_origin_url else None

    loader = SvnExportLoader(
        swh_storage,
        origin_url,
        svn_url=svn_url,
        ref=svn_revision,
        svn_paths=svn_paths,
        checksum_layout=checksum_layout,
        checksums=checksums,
    )

    actual_result = loader.load()

    assert actual_result == {"status": "eventful"}

    actual_visit = assert_last_visit_matches(
        swh_storage,
        origin_url,
        status="full",
        type="svn-export",
    )

    snapshot = swh_storage.snapshot_get(actual_visit.snapshot)
    assert snapshot is not None

    branches = snapshot["branches"].keys()
    expected_branch = f"rev_{svn_revision}".encode()
    assert set(branches) == {b"HEAD", expected_branch}

    assert get_stats(swh_storage) == {
        "content": 3,
        "directory": 5,
        "origin": 1,
        "origin_visit": 1,
        "release": 0,
        "revision": 0,
        "skipped_content": 0,
        "snapshot": 1,
    }

    # Ensure the extids got stored as well
    extids = fetch_extids_from_checksums(
        loader.storage,
        checksum_layout=checksum_layout,
        checksums=checksums,
        extid_version=loader.extid_version,
    )
    assert extids[0].extid.hex() == checksums["sha256"]

    # Another run should be uneventful
    loader2 = SvnExportLoader(
        swh_storage,
        origin_url,
        svn_url=svn_url,
        ref=svn_revision,
        svn_paths=svn_paths,
        checksum_layout=checksum_layout,
        checksums=checksums,
    )
    actual_result2 = loader2.load()
    assert actual_result2 == {"status": "uneventful"}


def test_loader_svn_directory_hash_mismatch(swh_storage, datadir, tmp_path):
    """Loading a svn tree with faulty checksums should fail"""
    archive_name = "pkg-gourmet"
    archive_path = os.path.join(datadir, f"{archive_name}.tgz")
    repo_url = prepare_repository_from_archive(
        archive_path, archive_name, tmp_path=tmp_path
    )
    faulty_checksums = {
        "sha256": "00000ed1855beadfa9c00f730242f5efe3e4612e76f0dcc45215c4a3234c7466"
    }
    loader = SvnExportLoader(
        swh_storage,
        repo_url,
        ref=5,
        checksum_layout="nar",
        checksums=faulty_checksums,
    )

    actual_result = loader.load()

    # Ingestion fails because the checks failed
    assert actual_result["status"] == "failed"
    assert get_stats(swh_storage) == {
        "content": 0,
        "directory": 0,
        "origin": 1,
        "origin_visit": 1,
        "release": 0,
        "revision": 0,
        "skipped_content": 0,
        "snapshot": 0,
    }

    # Ensure no extids got stored
    extids = fetch_extids_from_checksums(
        loader.storage,
        checksum_layout="nar",
        checksums=faulty_checksums,
        extid_version=loader.extid_version,
    )
    assert len(extids) == 0


def test_loader_svn_directory_not_found(swh_storage, datadir, tmp_path):
    """Loading a svn tree from an unknown origin should fail"""
    loader = SvnExportLoader(
        swh_storage,
        "file:///home/origin/does/not/exist",
        ref=5,
        checksum_layout="standard",
        checksums={},
    )

    actual_result = loader.load()

    # Ingestion fails because the checks failed
    assert actual_result == {"status": "uneventful"}
    assert get_stats(swh_storage) == {
        "content": 0,
        "directory": 0,
        "origin": 1,
        "origin_visit": 1,
        "release": 0,
        "revision": 0,
        "skipped_content": 0,
        "snapshot": 0,
    }


def test_loader_svn_directory_path_not_in_head_revision(swh_storage, datadir, tmp_path):
    """Check a directory path present in specific revision but not present in HEAD revision
    can be exported."""
    archive_name = "pkg-gourmet-add-remove-dir"
    archive_path = os.path.join(datadir, f"{archive_name}.tgz")
    repo_url = (
        prepare_repository_from_archive(archive_path, "pkg-gourmet", tmp_path=tmp_path)
        # path deleted in HEAD (revision 7)
        + "/gourmet/trunk/debian"
    )
    svn_revision = 6
    checksums = {"sha256": compute_nar_hash_for_rev(repo_url, svn_revision)}

    loader = SvnExportLoader(
        swh_storage,
        repo_url,
        ref=svn_revision,
        checksum_layout="nar",
        checksums=checksums,
    )

    actual_result = loader.load()

    assert actual_result == {"status": "eventful"}



# Copyright (C) 2023  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import pytest

from swh.loader.svn.fast_crawler import crawl_repository

from .utils import CommitChange, CommitChangeType, add_commit


def test_crawl_repository_bad_arg_types():
    with pytest.raises(TypeError):
        crawl_repository(repo_url=1)

    with pytest.raises(TypeError):
        crawl_repository("https://svn.example.org", revnum="1")


def test_crawl_repository_runtime_error():
    invalid_repo_url = "file:///tmp/svn/repo/not/found"
    with pytest.raises(
        RuntimeError,
        match=f"Unable to connect to a repository at URL '{invalid_repo_url}'",
    ):
        crawl_repository(invalid_repo_url)


def test_crawl_repository(repo_url):
    commits = [
        {
            "message": "Initial commit",
            "changes": [
                CommitChange(
                    change_type=CommitChangeType.AddOrUpdate,
                    path="code/hello/hello-world",
                    properties={"svn:executable": "*"},
                    data=b"#!/bin/bash\necho Hello World !",
                )
            ],
        },
        {
            "message": "Second commit",
            "changes": [
                CommitChange(
                    change_type=CommitChangeType.AddOrUpdate,
                    path="code/externals/",
                    properties={
                        "svn:externals": "https://svn.example.org/project project"
                    },
                )
            ],
        },
    ]

    for i, commit in enumerate(commits):
        add_commit(
            repo_url,
            commit["message"],
            commit["changes"],
        )

        paths = crawl_repository(repo_url, revnum=i + 1)
        for change in commit["changes"]:
            path = change["path"].rstrip("/")
            path_split = path.split("/")
            for i in range(len(path_split)):
                assert "/".join(path_split[: i + 1]) in paths
            for prop, value in change["properties"].items():
                assert paths[path]["props"][prop] == value

    assert crawl_repository(repo_url, revnum=len(commits)) == crawl_repository(repo_url)



# Copyright (C) 2015-2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information


from celery import shared_task

from swh.loader.core.utils import parse_visit_date

from .directory import SvnExportLoader
from .loader import SvnLoaderFromDump, SvnLoaderFromRemoteDump


def _process_kwargs(kwargs):
    if "visit_date" in kwargs:
        kwargs["visit_date"] = parse_visit_date(kwargs["visit_date"])
    return kwargs


@shared_task(name=__name__ + ".DumpMountAndLoadSvnRepository")
def load_svn(**kwargs):
    """
    1. Mount a remote svn dump as a local svn repository.
    2. Load it through the svn loader.
    3. Clean up mounted svn repository archive.
    """
    loader = SvnLoaderFromRemoteDump.from_configfile(**_process_kwargs(kwargs))
    return loader.load()


@shared_task(name=__name__ + ".MountAndLoadSvnRepository")
def load_svn_from_dump(**kwargs):
    """
    1. Mount a (possibly gzip compressed) svn dump as a local svn repository.
    2. Load it through the svn loader.
    """
    loader = SvnLoaderFromDump.from_configfile(**_process_kwargs(kwargs))
    return loader.load()


@shared_task(name=f"{__name__}.LoadSvnExport")
def load_svn_export(**kwargs):
    """Load svn tree into the swh archive."""
    loader = SvnExportLoader.from_configfile(**_process_kwargs(kwargs))
    return loader.load()



# Copyright (C) 2019-2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

from contextlib import closing
import socket
import subprocess
import time
from typing import Any, Dict
import uuid

import pytest

from swh.loader.svn.loader import SvnLoader, SvnLoaderFromRemoteDump
from swh.scheduler.model import Lister

from .utils import create_repo

NAMESPACE = "swh.loader.svn"


@pytest.fixture(params=[SvnLoader, SvnLoaderFromRemoteDump])
def svn_loader_cls(request):
    return request.param


@pytest.fixture
def swh_storage_backend_config(swh_storage_backend_config):
    """Basic pg storage configuration with no journal collaborator
    (to avoid pulling optional dependency on clients of this fixture)

    """
    return {
        "cls": "filter",
        "storage": {
            "cls": "buffer",
            "min_batch_size": {
                "content": 10000,
                "content_bytes": 1073741824,
                "directory": 2500,
                "revision": 10,
                "release": 100,
            },
            "storage": swh_storage_backend_config,
        },
    }


@pytest.fixture
def swh_loader_config(swh_storage_backend_config) -> Dict[str, Any]:
    return {
        "storage": swh_storage_backend_config,
        "check_revision": 100,
        "temp_directory": "/tmp",
    }


@pytest.fixture
def repo_url(tmpdir_factory):
    # create a repository
    return create_repo(tmpdir_factory.mktemp("repos"))


@pytest.fixture(autouse=True)
def mock_sleep(mocker):
    return mocker.patch("time.sleep")


def find_free_port():
    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
        s.bind(("", 0))
        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        return s.getsockname()[1]


# https://gist.github.com/butla/2d9a4c0f35ea47b7452156c96a4e7b12
def wait_for_port(port: int, host: str = "localhost", timeout: float = 5.0):
    start_time = time.perf_counter()
    while True:
        try:
            with socket.create_connection((host, port), timeout=timeout):
                break
        except OSError as ex:
            time.sleep(0.01)
            if time.perf_counter() - start_time >= timeout:
                raise TimeoutError(
                    f"Waited too long for the port {port} on host {host} "
                    "to start accepting connections."
                ) from ex


@pytest.fixture
def svnserve():
    """Fixture wrapping svnserve execution and ensuring to terminate it
    after test run"""
    svnserve_proc = None

    def run_svnserve(repo_root):
        nonlocal svnserve_proc
        port = find_free_port()
        svnserve_proc = subprocess.Popen(
            [
                "svnserve",
                "-d",
                "--foreground",
                "--listen-port",
                str(port),
                "--root",
                repo_root,
            ]
        )
        wait_for_port(port)
        return port

    yield run_svnserve

    svnserve_proc.terminate()


@pytest.fixture
def svn_lister():
    return Lister(name="svn-lister", instance_name="example", id=uuid.uuid4())



# Copyright (C) 2022-2023  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

from datetime import datetime
from enum import Enum
from io import BytesIO
import os
from typing import Dict, List, Optional

from subvertpy import SubversionException, delta, repos
from subvertpy.ra import Auth, RemoteAccess, get_username_provider
from typing_extensions import TypedDict


class CommitChangeType(Enum):
    AddOrUpdate = 1
    Delete = 2


class CommitChange(TypedDict, total=False):
    change_type: CommitChangeType
    path: str
    properties: Dict[str, str]
    data: bytes
    copyfrom_path: str
    copyfrom_rev: int


def add_commit(
    repo_url: str,
    message: str,
    changes: List[CommitChange],
    date: Optional[datetime] = None,
) -> None:
    conn = RemoteAccess(repo_url, auth=Auth([get_username_provider()]))
    editor = conn.get_commit_editor({"svn:log": message})
    root = editor.open_root()
    for change in changes:
        if change["change_type"] == CommitChangeType.Delete:
            root.delete_entry(change["path"].rstrip("/"))
        else:
            dir_change = change["path"].endswith("/")
            split_path = change["path"].rstrip("/").split("/")
            copyfrom_path = change.get("copyfrom_path")
            copyfrom_rev = change.get("copyfrom_rev", -1)
            for i in range(len(split_path)):
                path = "/".join(split_path[0 : i + 1])
                if i < len(split_path) - 1:
                    try:
                        root.add_directory(path, copyfrom_path, copyfrom_rev).close()
                    except SubversionException:
                        pass
                else:
                    if dir_change:
                        try:
                            dir = root.add_directory(path, copyfrom_path, copyfrom_rev)
                        except SubversionException:
                            dir = root.open_directory(path)
                        if "properties" in change:
                            for prop, value in change["properties"].items():
                                dir.change_prop(prop, value)
                        dir.close()
                    else:
                        try:
                            file = root.add_file(path, copyfrom_path, copyfrom_rev)
                        except SubversionException:
                            file = root.open_file(path)
                        if "properties" in change:
                            for prop, value in change["properties"].items():
                                file.change_prop(prop, value)
                        if "data" in change:
                            txdelta = file.apply_textdelta()
                            delta.send_stream(BytesIO(change["data"]), txdelta)
                        file.close()
    root.close()
    editor.close()

    if date is not None:
        conn.change_rev_prop(
            conn.get_latest_revnum(),
            "svn:date",
            date.strftime("%Y-%m-%dT%H:%M:%S.%fZ").encode(),
        )


def create_repo(tmp_path, repo_name="tmprepo"):
    repo_path = os.path.join(tmp_path, repo_name)
    repos.create(repo_path)
    # add passthrough hooks to allow modifying revision properties like svn:date
    hooks_path = f"{repo_path}/hooks"
    for hook_file in (
        f"{hooks_path}/pre-revprop-change",
        f"{hooks_path}/post-revprop-change",
    ):
        with open(hook_file, "wb") as hook:
            hook.write(b"#!/bin/sh\n\nexit 0")
        os.chmod(hook_file, 0o775)
    return f"file://{repo_path}"



# Copyright (C) 2023  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

from typing import Dict

from typing_extensions import TypedDict

class SvnPathInfo(TypedDict):
    type: str
    props: Dict[str, str]

def crawl_repository(
    repo_url: str, revnum: int = -1, username: str = "", password: str = ""
) -> Dict[str, SvnPathInfo]: ...



# Copyright (C) 2022  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

from typing import Any, Dict

import pytest

from swh.scheduler.model import ListedOrigin

from .conftest import NAMESPACE


@pytest.fixture
def swh_loader_config(swh_storage_backend_config) -> Dict[str, Any]:
    return {
        "storage": swh_storage_backend_config,
    }


@pytest.fixture
def svn_listed_svn_directory_origin(svn_lister):
    return ListedOrigin(
        lister_id=svn_lister.id,
        url="svn://example.org/repo",
        visit_type="directory",
    )


@pytest.mark.parametrize(
    "extra_loader_arguments",
    [
        {"checksum_layout": "nar", "checksums": {}, "ref": "5"},
        {"checksum_layout": "standard", "checksums": {}, "ref": "6"},
    ],
)
def test_svn_directory_loader_for_listed_origin(
    loading_task_creation_for_listed_origin_test,
    svn_lister,
    svn_listed_svn_directory_origin,
    extra_loader_arguments,
):
    svn_listed_svn_directory_origin.extra_loader_arguments = extra_loader_arguments

    loading_task_creation_for_listed_origin_test(
        loader_class_name=f"{NAMESPACE}.directory.SvnExportLoader",
        task_function_name=f"{NAMESPACE}.tasks.LoadSvnExport",
        lister=svn_lister,
        listed_origin=svn_listed_svn_directory_origin,
    )



#!/usr/bin/env python3
# Copyright (C) 2015-2023  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import errno
import os
import shlex
import subprocess

from setuptools import Extension, setup

##############################################################################################
# The code below is taken from the setup.py file of the subvertpy project by Jelmer Vernooij
# https://github.com/jelmer/subvertpy/blob/9a3d963e6cea8480e3efa06d78c50980769ce486/setup.py
# License: GNU Lesser General Public License v2.1


def config_value(command, env, args):
    command = os.environ.get(env, command)
    try:
        return subprocess.check_output([command] + args).strip().decode()
    except OSError as e:
        if e.errno == errno.ENOENT:
            raise Exception(
                "%s not found. Please set %s environment variable" % (command, env)
            )
        raise


def split_shell_results(line):
    return shlex.split(line)


def apr_config(args):
    return config_value("apr-1-config", "APR_CONFIG", args)


def apr_build_data():
    """Determine the APR header file location."""
    try:
        includedir = os.environ["APR_INCLUDE_DIR"]
    except KeyError:
        includedir = apr_config(["--includedir"])
    if not os.path.isdir(includedir):
        raise Exception("APR development headers not found")
    try:
        extra_link_flags = split_shell_results(os.environ["APR_LINK_FLAGS"])
    except KeyError:
        extra_link_flags = split_shell_results(apr_config(["--link-ld", "--libs"]))
    return (includedir, extra_link_flags)


def svn_build_data():
    """Determine the Subversion header file location."""
    if "SVN_HEADER_PATH" in os.environ and "SVN_LIBRARY_PATH" in os.environ:
        return ([os.getenv("SVN_HEADER_PATH")], [os.getenv("SVN_LIBRARY_PATH")], [])
    svn_prefix = os.getenv("SVN_PREFIX")
    if svn_prefix is None:
        basedirs = ["/usr/local", "/usr"]
        for basedir in basedirs:
            includedir = os.path.join(basedir, "include/subversion-1")
            if os.path.isdir(includedir):
                svn_prefix = basedir
                break
    if svn_prefix is not None:
        return (
            [os.path.join(svn_prefix, "include/subversion-1")],
            [os.path.join(svn_prefix, "lib")],
            [],
        )
    raise Exception(
        "Subversion development files not found. "
        "Please set SVN_PREFIX or (SVN_LIBRARY_PATH and "
        "SVN_HEADER_PATH) environment variable. "
    )


(apr_includedir, apr_link_flags) = apr_build_data()
(svn_includedirs, svn_libdirs, svn_link_flags) = svn_build_data()


class SvnExtension(Extension):
    def __init__(self, name, *args, **kwargs):
        kwargs["include_dirs"] = [apr_includedir] + svn_includedirs
        kwargs["library_dirs"] = svn_libdirs
        kwargs["extra_link_args"] = apr_link_flags + svn_link_flags
        Extension.__init__(self, name, *args, **kwargs)


# end of code taken from subvertpy project
##############################################################################################

setup(
    ext_modules=[
        SvnExtension(
            "swh.loader.svn.fast_crawler",
            ["swh/loader/svn/fast_crawler.cpp"],
            libraries=["svn_delta-1", "svn_ra-1", "svn_subr-1"],
        )
    ],
)
